@@ -3,12 +3,12 @@
  * rtree.c
  *	  interface routines for the postgres rtree indexed access method.
  *
- * Portions Copyright (c) 1996-2001, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2003, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /cvsroot/pgsql/src/backend/access/rtree/rtree.c,v 1.69 2002/01/15 22:14:17 tgl Exp $
+ *	  $Header: /cvsroot/pgsql/src/backend/access/rtree/rtree.c,v 1.80 2003/09/25 06:57:57 petere Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -125,7 +125,7 @@ rtbuild(PG_FUNCTION_ARGS)
 	 * that's not the case, big trouble's what we have.
 	 */
 	if (RelationGetNumberOfBlocks(index) != 0)
-		elog(ERROR, "%s already contains data",
+		elog(ERROR, "index \"%s\" already contains data",
 			 RelationGetRelationName(index));
 
 	/* initialize the root page */
@@ -226,6 +226,7 @@ rtinsert(PG_FUNCTION_ARGS)
 
 #ifdef NOT_USED
 	Relation	heapRel = (Relation) PG_GETARG_POINTER(4);
+	bool		checkUnique = PG_GETARG_BOOL(5);
 #endif
 	InsertIndexResult res;
 	IndexTuple	itup;
@@ -327,7 +328,7 @@ rtdoinsert(Relation r, IndexTuple itup, RTSTATE *rtstate)
 						LP_USED);
 	}
 	if (l == InvalidOffsetNumber)
-		elog(ERROR, "rtdoinsert: failed to add index item to %s",
+		elog(ERROR, "failed to add index item to \"%s\"",
 			 RelationGetRelationName(r));
 
 	WriteBuffer(buffer);
@@ -519,7 +520,7 @@ rtdosplit(Relation r,
 
 		if (PageAddItem(left, (Item) item, IndexTupleSize(item),
 						leftoff, LP_USED) == InvalidOffsetNumber)
-			elog(ERROR, "rtdosplit: failed to add index item to %s",
+			elog(ERROR, "failed to add index item to \"%s\"",
 				 RelationGetRelationName(r));
 		leftoff = OffsetNumberNext(leftoff);
 
@@ -543,7 +544,7 @@ rtdosplit(Relation r,
 
 		if (PageAddItem(right, (Item) item, IndexTupleSize(item),
 						rightoff, LP_USED) == InvalidOffsetNumber)
-			elog(ERROR, "rtdosplit: failed to add index item to %s",
+			elog(ERROR, "failed to add index item to \"%s\"",
 				 RelationGetRelationName(r));
 		rightoff = OffsetNumberNext(rightoff);
 
@@ -639,7 +640,9 @@ rtintinsert(Relation r,
 	 */
 
 	if (IndexTupleSize(old) != IndexTupleSize(ltup))
-		elog(ERROR, "Variable-length rtree keys are not supported.");
+		ereport(ERROR,
+				(errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
+				 errmsg("variable-length rtree keys are not supported")));
 
 	/* install pointer to left child */
 	memmove(old, ltup, IndexTupleSize(ltup));
@@ -659,7 +662,7 @@ rtintinsert(Relation r,
 		if (PageAddItem(p, (Item) rtup, IndexTupleSize(rtup),
 						PageGetMaxOffsetNumber(p),
 						LP_USED) == InvalidOffsetNumber)
-			elog(ERROR, "rtintinsert: failed to add index item to %s",
+			elog(ERROR, "failed to add index item to \"%s\"",
 				 RelationGetRelationName(r));
 		WriteBuffer(b);
 		ldatum = IndexTupleGetDatum(ltup);
@@ -685,12 +688,12 @@ rtnewroot(Relation r, IndexTuple lt, IndexTuple rt)
 	if (PageAddItem(p, (Item) lt, IndexTupleSize(lt),
 					FirstOffsetNumber,
 					LP_USED) == InvalidOffsetNumber)
-		elog(ERROR, "rtnewroot: failed to add index item to %s",
+		elog(ERROR, "failed to add index item to \"%s\"",
 			 RelationGetRelationName(r));
 	if (PageAddItem(p, (Item) rt, IndexTupleSize(rt),
 					OffsetNumberNext(FirstOffsetNumber),
 					LP_USED) == InvalidOffsetNumber)
-		elog(ERROR, "rtnewroot: failed to add index item to %s",
+		elog(ERROR, "failed to add index item to \"%s\"",
 			 RelationGetRelationName(r));
 	WriteBuffer(b);
 }
@@ -777,8 +780,11 @@ rtpicksplit(Relation r,
 	 */
 	newitemsz = IndexTupleTotalSize(itup);
 	if (newitemsz > RTPageAvailSpace)
-		elog(ERROR, "rtree: index item size %lu exceeds maximum %lu",
-			 (unsigned long) newitemsz, (unsigned long) RTPageAvailSpace);
+		ereport(ERROR,
+				(errcode(ERRCODE_PROGRAM_LIMIT_EXCEEDED),
+				 errmsg("index row size %lu exceeds rtree maximum, %lu",
+						(unsigned long) newitemsz,
+						(unsigned long) RTPageAvailSpace)));
 
 	maxoff = PageGetMaxOffsetNumber(page);
 	newitemoff = OffsetNumberNext(maxoff);		/* phony index for new
@@ -931,6 +937,8 @@ rtpicksplit(Relation r,
 						  PointerGetDatum(&size_alpha));
 			FunctionCall2(&rtstate->sizeFn, union_dr,
 						  PointerGetDatum(&size_beta));
+			pfree(DatumGetPointer(union_dl));
+			pfree(DatumGetPointer(union_dr));
 
 			diff = (size_alpha - size_l) - (size_beta - size_r);
 
@@ -1062,7 +1070,7 @@ rtpicksplit(Relation r,
 			choose_left = false;
 		else
 		{
-			elog(ERROR, "rtpicksplit: failed to find a workable page split");
+			elog(ERROR, "failed to find a workable rtree page split");
 			choose_left = false;	/* keep compiler quiet */
 		}
 
@@ -1189,7 +1197,6 @@ rtbulkdelete(PG_FUNCTION_ARGS)
 	BlockNumber num_pages;
 	double		tuples_removed;
 	double		num_index_tuples;
-	RetrieveIndexResult res;
 	IndexScanDesc iscan;
 
 	tuples_removed = 0;
@@ -1206,23 +1213,22 @@ rtbulkdelete(PG_FUNCTION_ARGS)
 	 */
 
 	/* walk through the entire index */
-	iscan = index_beginscan(rel, false, 0, (ScanKey) NULL);
+	iscan = index_beginscan(NULL, rel, SnapshotAny, 0, (ScanKey) NULL);
+	/* including killed tuples */
+	iscan->ignore_killed_tuples = false;
 
-	while ((res = index_getnext(iscan, ForwardScanDirection))
-		   != (RetrieveIndexResult) NULL)
+	while (index_getnext_indexitem(iscan, ForwardScanDirection))
 	{
-		ItemPointer heapptr = &res->heap_iptr;
-
-		if (callback(heapptr, callback_state))
+		if (callback(&iscan->xs_ctup.t_self, callback_state))
 		{
-			ItemPointer indexptr = &res->index_iptr;
+			ItemPointerData indextup = iscan->currentItemData;
 			BlockNumber blkno;
 			OffsetNumber offnum;
 			Buffer		buf;
 			Page		page;
 
-			blkno = ItemPointerGetBlockNumber(indexptr);
-			offnum = ItemPointerGetOffsetNumber(indexptr);
+			blkno = ItemPointerGetBlockNumber(&indextup);
+			offnum = ItemPointerGetOffsetNumber(&indextup);
 
 			/* adjust any scans that will be affected by this deletion */
 			/* (namely, my own scan) */
@@ -1240,8 +1246,6 @@ rtbulkdelete(PG_FUNCTION_ARGS)
 		}
 		else
 			num_index_tuples += 1;
-
-		pfree(res);
 	}
 
 	index_endscan(iscan);
@@ -1249,10 +1253,10 @@ rtbulkdelete(PG_FUNCTION_ARGS)
 	/* return statistics */
 	num_pages = RelationGetNumberOfBlocks(rel);
 
-	result = (IndexBulkDeleteResult *) palloc(sizeof(IndexBulkDeleteResult));
+	result = (IndexBulkDeleteResult *) palloc0(sizeof(IndexBulkDeleteResult));
 	result->num_pages = num_pages;
-	result->tuples_removed = tuples_removed;
 	result->num_index_tuples = num_index_tuples;
+	result->tuples_removed = tuples_removed;
 
 	PG_RETURN_POINTER(result);
 }
@@ -1345,13 +1349,13 @@ _rtdump(Relation r)
 void
 rtree_redo(XLogRecPtr lsn, XLogRecord *record)
 {
-	elog(STOP, "rtree_redo: unimplemented");
+	elog(PANIC, "rtree_redo: unimplemented");
 }
 
 void
 rtree_undo(XLogRecPtr lsn, XLogRecord *record)
 {
-	elog(STOP, "rtree_undo: unimplemented");
+	elog(PANIC, "rtree_undo: unimplemented");
 }
 
 void
